% From the book
% PROLOG PROGRAMMING IN DEPTH
% by Michael A. Covington, Donald Nute, and Andre Vellino
% (Prentice Hall, 1997).
% Copyright 1997 Prentice-Hall, Inc.
% For educational use only

% File CAR.PL
% Simple automotive expert system

%
% Main control procedures
%

start :-
   %write('This program diagnoses why a car won''t start.'),nl,
   %write('Answer all questions with yes or no.'),nl,
   clear_stored_answers,
   try_all_possibilities.

try_all_possibilities :-     % Backtrack through all possibilities...
   defect_may_be(D),
   explain(D),
   fail.

try_all_possibilities.       % ...then succeed with no further action.


%
% Diagnostic knowledge base
%   (conditions under which to give each diagnosis)
%

defect_may_be(drained_battery) :-
   user_says(starter_was_ok,yes),
   user_says(starter_is_ok,no).

defect_may_be(wrong_gear) :-
   user_says(starter_was_ok,no).

defect_may_be(starting_system) :-
   user_says(starter_was_ok,no).

defect_may_be(fuel_system) :-
   user_says(starter_was_ok,yes),
   user_says(fuel_is_ok,no).

defect_may_be(ignition_system) :-
   user_says(starter_was_ok,yes),
   user_says(fuel_is_ok,yes).


%
% Case knowledge base
%   (information supplied by the user during the consultation)
%

:- dynamic(stored_answer/2).

   % (Clauses get added as user answers questions.)


%
% Procedure to get rid of the stored answers
% without abolishing the dynamic declaration
%

clear_stored_answers :- retract(stored_answer(_,_)),fail.
clear_stored_answers.


%
% Procedure to retrieve the user's answer to each question when needed,
% or ask the question if it has not already been asked
%

user_says(Q,A) :- stored_answer(Q,A).

user_says(Q,A) :- \+ stored_answer(Q,_),
                  nl,nl,
                  ask_question(Q,Response),
                  asserta(stored_answer(Q,Response)),
                  Response = A.


%
% Texts of the questions
%

ask_question(starter_was_ok,R) :-
   Q = 'When you first started trying to start the car,\ndid the starter crank the engine normally?',
   T = 'Question1',
   jpl_call('javax.swing.JOptionPane',showConfirmDialog,[@(null),Q,T,0],A),
   (	A == 0
   ->	R = 'yes'
   ;	R = 'no'
   ).

ask_question(starter_is_ok,R) :-
   Q = 'Does the starter crank the engine normally now?',
   T = 'Question2',
   jpl_call('javax.swing.JOptionPane',showConfirmDialog,[@(null),Q,T,0],A),
   (	A == 0
   ->	R = 'yes'
   ;	R = 'no'
   ).

ask_question(fuel_is_ok,R) :-
   Q = 'Look in the carburetor. Can you see or smell gasoline?',
   T = 'Question3',
   jpl_call('javax.swing.JOptionPane',showConfirmDialog,[@(null),Q,T,0],A),
   (	A == 0
   ->	R = 'yes'
   ;	R = 'no'
   ).

%
%  Explanations for the various diagnoses
%

explain(wrong_gear) :-
   Q = 'Check that the gearshift is set to Park or Neutral.\nTry jiggling the gearshift lever.',
   T = 'Wrong gear',
   jpl_call('javax.swing.JOptionPane',showMessageDialog,[@(null),Q,T,1],_).

explain(starting_system) :-
   Q = 'Check for a defective battery, voltage\nregulator, or alternator; if any of these is\nthe problem, charging the battery or jump-\nstarting may get the car going temporarily.\nOr the starter itself may be defective.',
   T = 'Starting system',
   jpl_call('javax.swing.JOptionPane',showMessageDialog,[@(null),Q,T,1],_).

explain(drained_battery) :-
   Q = 'Your attempts to start the car have run down the battery.\nRecharging or jump-starting will be necessary.\nBut there is probably nothing wrong with the battery itself.',
   T = 'Starting system',
   jpl_call('javax.swing.JOptionPane',showMessageDialog,[@(null),Q,T,1],_).

explain(fuel_system) :-
   Q = 'Check whether there is fuel in the tank.\nIf so, check for a clogged fuel line or filter\nor a defective fuel pump.',
   T = 'Fuel system',
   jpl_call('javax.swing.JOptionPane',showMessageDialog,[@(null),Q,T,1],_).

explain(ignition_system) :-
   Q = 'Check the spark plugs, cables, distributor,\ncoil, and other parts of the ignition system.\nIf any of these are visibly defective or long\noverdue for replacement, replace them; if this\ndoes not solve the problem, consult a mechanic.',
   T = 'Ignition system',
   jpl_call('javax.swing.JOptionPane',showMessageDialog,[@(null),Q,T,1],_).


